@@ -41,22 +41,16 @@
 (defun generate-call-sequence (name style vop options)
   (declare (ignore options))
   (ecase style
-      (:raw
-       (values
-        `((note-this-location ,vop :call-site)
-          (invoke-asm-routine 'call ',name ,vop)
-          (note-this-location ,vop :single-value-return))
-        nil))
-      ((:full-call :full-call-no-return)
-       (values
-        `((note-this-location ,vop :call-site)
-          (invoke-asm-routine 'call ',name ,vop)
-          (note-this-location ,vop :single-value-return))
-        '((:save-p :compute-only))))
-      (:none
-       (values
-        `((invoke-asm-routine 'jmp ',name ,vop))
-        nil))))
+    ((:raw :full-call :full-call-no-return)
+     (values
+      `((note-this-location ,vop :call-site)
+        (invoke-asm-routine 'call ',name ,vop)
+        (note-this-location ,vop :single-value-return))
+      (if (eql style :raw) nil '((:save-p :compute-only)))))
+    (:none
+     (values
+      `((invoke-asm-routine 'jmp ',name ,vop))
+      nil))))
 
 (defun generate-return-sequence (style)
   (ecase style
@@ -99,7 +93,7 @@
        ;; KLUDGE: index of FPR-SAVE is 4
        ;; (inst call (ea (make-fixup 'fpr-save :assembly-routine*)))
        (inst call (ea (make-fixup nil :code-object
-                                  (+ (ash code-constants-offset word-shift)
+                                  (+ (component-header-length)
                                      (* 4 sb-vm:n-word-bytes)
                                      (- other-pointer-lowtag)))
                       rip-tn))
@@ -109,7 +103,7 @@
        ;; KLUDGE: index of FPR-RESTORE is 6
        ;; (inst call (ea (make-fixup 'fpr-restore :assembly-routine*)))
        (inst call (ea (make-fixup nil :code-object
-                                  (+ (ash code-constants-offset word-shift)
+                                  (+ (component-header-length)
                                      (* 6 sb-vm:n-word-bytes)
                                      (- other-pointer-lowtag)))
                       rip-tn))
