@@ -136,6 +136,11 @@
   (partial-source-info nil :type (or null sb-c::debug-source)))
 (declaim (freeze-type fasl-input))
 
+;;; Lisp assembler routines are named by Lisp symbols, not strings,
+;;; and so can be compared by EQ.
+(define-load-time-global *assembler-routines* nil)
+#-sb-xc-host (declaim (code-component *assembler-routines*))
+
 ;;; Output the current number of semicolons after a fresh-line.
 ;;; FIXME: non-mnemonic name
 (defun load-fresh-line ()
@@ -439,7 +444,8 @@
     (when (functionp function)
       (let ((oname (nth-value 2 (function-lambda-expression function))))
         (when (and oname (not (eq oname name)))
-          (error "fop ~S with opcode ~D conflicts with fop ~S."
+          (cerror "Define it anyway"
+                  "fop ~S with opcode ~D conflicts with fop ~S."
                  name opcode oname))))
     (let ((existing-opcode (get name 'opcode)))
       (when (and existing-opcode (/= existing-opcode opcode))
@@ -1081,57 +1087,102 @@
 ;;; putting the implementation and version in required fields in the
 ;;; fasl file header.)
 
+;;; Caution: don't try to "test" WITH-WRITABLE-CODE-INSTRUCTIONS in copy-in/out mode
+;;; on any architecture where fixup application cares what the address of the code actually is.
+;;; This means x86 is disqualified. You're just wasting your time if you try, as I did.
+(defmacro with-writable-code-instructions ((code-var total-nwords debug-info-var
+                                            n-fdefns n-funs)
+                                           &key copy fixup)
+  (declare (ignorable n-fdefns))
+  ;; N-FDEFNS is important for PPC64, slightly important for X86-64, not important for
+  ;; any others, and doesn't even have a place to store it if lispwords are 32 bits.
+  ;; The :DEDUPLICATED-FDEFNS test in compiler-2.pure asserts that the value is valid
+  (let ((body
+         ;; The following operations need the code pinned:
+         ;; 1. copying into code-instructions (a SAP)
+         ;; 2. apply-core-fixups and sanctify-for-execution
+         ;; A very specific store order is necessary to allow using uninitialized memory
+         ;; pages for code. Storing of the debug-info slot must occur between steps 1 and 2.
+         ;; Note that this does not have to take care to ensure atomicity
+         ;; of the store to the final word of unboxed data. Even if BYTE-BLT were
+         ;; interrupted in between the store of any individual byte, this code
+         ;; is GC-safe because we no longer need to know where simple-funs are embedded
+         ;; within the object to trace pointers. We *do* need to know where the funs
+         ;; are when transporting the object, but it's pinned within the body forms.
+         `(,copy
+           (sb-c::code-header/trailer-adjust ,code-var ,total-nwords ,n-fdefns)
+           ;; Check that the code trailer matches our expectation on number of embedded simple-funs
+           (aver (= (code-n-entries ,code-var) ,n-funs))
+           ;; Until debug-info is assigned, it is illegal to create a simple-fun pointer
+           ;; into this object, because the C code assumes that the fun table is in an
+           ;; invalid/incomplete state (i.e. can't be read) until the code has debug-info.
+           ;; That is, C code can't deal with an interior code pointer until the fun-table
+           ;; is valid. This store must occur prior to calling %CODE-ENTRY-POINT, and
+           ;; applying fixups calls %CODE-ENTRY-POINT, so we have to do this before that.
+           (setf (%code-debug-info ,code-var) ,debug-info-var)
+           ,fixup)))
+    #+darwin-jit
+    `(with-pinned-objects (,code-var ,debug-info-var)
+       ;; DEBUG-INFO is pinned so that after assigning it into the temporary
+       ;; block of memory, the off-heap word which is invisible to GC remains valid.
+       (let* ((temp-copy (alien-funcall (extern-alien "duplicate_codeblob_offheap"
+                                                      (function unsigned unsigned))
+                                        (get-lisp-obj-address ,code-var)))
+              (aligned (+ temp-copy (logand temp-copy sb-vm:n-word-bytes))))
+         ;; Rebind CODE-VAR to the replica, then execute BODY
+         (let ((,code-var (%make-lisp-obj (logior aligned sb-vm:other-pointer-lowtag)))) ,@body)
+         ;; Copy back, and fixup the simple-funs in the managed object
+         (alien-funcall (extern-alien "jit_copy_code_insts" (function void unsigned unsigned))
+                        (get-lisp-obj-address ,code-var)
+                        temp-copy)))
+    #-darwin-jit
+    `(with-pinned-objects (,code-var) ,@body)))
+
 (define-load-time-global *show-new-code* nil)
-(define-fop 17 :not-host (fop-load-code ((:operands header n-code-bytes n-fixups)))
+(define-fop 17 :not-host (fop-load-code ((:operands header n-code-bytes n-fixup-elts)))
+  ;; The stack looks like:
+  ;; ... | constant0 constant1 ... constantN | DEBUG-INFO | FIXUPS-ITEMS ....   ||
+  ;;     | <--------- n-constants ---------> |            | <-- n-fixup-elts -> ||
   (let* ((n-simple-funs (read-unsigned-byte-32-arg (fasl-input-stream)))
          (n-named-calls (read-unsigned-byte-32-arg (fasl-input-stream)))
          (n-boxed-words (ash header -1))
-         (n-constants (- n-boxed-words sb-vm:code-constants-offset)))
-    ;; stack has (at least) N-CONSTANTS words plus debug-info
-    (with-fop-stack ((stack (operand-stack)) ptr (1+ n-constants))
+         (n-constants (- n-boxed-words sb-vm:code-constants-offset))
+         (stack-elts-consumed (+ n-constants 1 n-fixup-elts)))
+    (with-fop-stack ((stack (operand-stack)) ptr stack-elts-consumed)
       ;; We've already ensured that all FDEFNs the code uses exist.
       ;; This happened by virtue of calling fop-fdefn for each.
       (let ((stack-index (+ ptr (* n-simple-funs sb-vm:code-slots-per-simple-fun))))
         (dotimes (i n-named-calls)
           (aver (typep (svref stack stack-index) 'fdefn))
           (incf stack-index)))
-      (let ((code (sb-c:allocate-code-object
+      (binding* (((code total-nwords)
+                  (sb-c:allocate-code-object
                    (if (oddp header) :immobile :dynamic)
-                   n-named-calls
                    (align-up n-boxed-words sb-c::code-boxed-words-align)
-                   n-code-bytes)))
-        (with-pinned-objects (code)
-          ;; * DO * NOT * SEPARATE * THESE * STEPS *
-          ;; For a full explanation, refer to the comment above MAKE-CORE-COMPONENT
-          ;; concerning the corresponding use therein of WITH-PINNED-OBJECTS etc.
-          #-darwin-jit
-          (read-n-bytes (fasl-input-stream) (code-instructions code) 0 n-code-bytes)
-          #+darwin-jit
-          (let ((buf (make-array n-code-bytes :element-type '(unsigned-byte 8))))
-            (read-n-bytes (fasl-input-stream) buf 0 n-code-bytes)
-            (with-pinned-objects (buf)
-              (sb-vm::jit-memcpy (code-instructions code) (vector-sap buf) n-code-bytes)))
-          (aver (= (code-n-entries code) n-simple-funs))
-          ;; Serial# shares a word with the jump-table word count,
-          ;; so we can't assign serial# until after all raw bytes are copied in.
-          (sb-c::assign-code-serialno code)
-          (sb-thread:barrier (:write))
-          ;; Assign debug-info last. A code object that has no debug-info will never
-          ;; have its fun table accessed in conservative_root_p() or pin_object().
-          (setf (%code-debug-info code) (svref stack (+ ptr n-constants)))
+                   n-code-bytes))
+                 (real-code code)
+                 (debug-info (svref stack (+ ptr n-constants))))
+        (with-writable-code-instructions
+            (code total-nwords debug-info n-named-calls n-simple-funs)
+          :copy (read-n-bytes (fasl-input-stream) (code-instructions code) 0 n-code-bytes)
+          :fixup (sb-c::apply-fasl-fixups code stack (+ ptr (1+ n-constants)) n-fixup-elts real-code))
+        ;; Don't need the code pinned from here on
+        (progn
           (setf (sb-c::debug-info-source (%code-debug-info code))
                 (%fasl-input-partial-source-info (fasl-input)))
           ;; Boxed constants can be assigned only after figuring out where the range
           ;; of implicitly tagged words is, which requires knowing how many functions
           ;; are in the code component, which requires reading the code trailer.
+          #+darwin-jit (sb-c::assign-code-constants
+                        code (subseq stack ptr (+ ptr n-constants)))
+          #-darwin-jit
           (let* ((header-index sb-vm:code-constants-offset)
                  (stack-index ptr))
             (declare (type index header-index stack-index))
-            (dotimes (n n-simple-funs)
-              (dotimes (i sb-vm:code-slots-per-simple-fun)
-                (setf (code-header-ref code header-index) (svref stack stack-index))
-                (incf header-index)
-                (incf stack-index)))
+            (dotimes (n (* n-simple-funs sb-vm:code-slots-per-simple-fun))
+              (setf (code-header-ref code header-index) (svref stack stack-index))
+              (incf header-index)
+              (incf stack-index))
             (dotimes (i n-named-calls)
               (sb-c::set-code-fdefn code header-index (svref stack stack-index))
               (incf header-index)
@@ -1139,13 +1190,10 @@
             (do () ((>= header-index n-boxed-words))
               (setf (code-header-ref code header-index) (svref stack stack-index))
               (incf header-index)
-              (incf stack-index)))
-          ;; Now apply fixups. The fixups to perform are popped from the fasl stack.
-          (sb-c::apply-fasl-fixups stack code n-fixups))
+              (incf stack-index))))
         (when *show-new-code*
           (let ((*print-pretty* nil))
             (format t "~&New code(~Db,load): ~A~%" (code-object-size code) code)))
-        #-sb-xc-host
         (when (typep (code-header-ref code (1- n-boxed-words))
                      '(cons (eql sb-c::coverage-map)))
           ;; Record this in the global list of coverage-instrumented code.
@@ -1238,9 +1286,9 @@
                     ,@(mapcar (lambda (spec) `((,(cadr spec)) ,(car spec)))
                               specs)))
                 ,@(mapcar (lambda (spec)
-                            `(define-fop ,(car spec)
-                                          (,(symbolicate "FOP-LAYOUT-OF-"
-                                                         (cadr spec)))
+                            `(define-fop ,(car spec) :not-host
+                               (,(symbolicate "FOP-LAYOUT-OF-"
+                                              (cadr spec)))
                                ,(find-layout (cadr spec))))
                           specs))))
   (frob (#x68 t)
@@ -1298,4 +1346,3 @@
       (dolist (m times)
         (format t "~30S: ~6,2F~%" (car m) (/ (float (cdr m)) 60.0))))))
 |#
-
