@@ -316,10 +316,29 @@
   (declare (explicit-check))
   (macrolet ((truncate-float (rtype)
                `(let* ((float-div (coerce divisor ',rtype))
-                       (res (%unary-truncate (/ number float-div))))
+                       (divided (/ number float-div))
+                       (res (%unary-truncate divided)))
                   (values res
                           (- number
-                             (* (coerce res ',rtype) float-div))))))
+                             (* #-round-float
+                                (coerce res ',rtype)
+                                #+round-float
+                                (,(ecase rtype
+                                    (double-float 'round-double)
+                                    (single-float 'round-single))
+                                 divided :truncate)
+                                float-div)))))
+             (single-digit-bignum-p (x)
+               #+(or x86-64 x86 ppc64)
+               `(or (typep ,x 'word)
+                    (typep ,x 'sb-vm:signed-word))
+               ;; Other backends don't have native double-word/word division,
+               ;; and their bigfloor implementation doesn't handle
+               ;; full-width divisors.
+               #-(or x86-64 x86 ppc64)
+               `(or (typep ,x '(unsigned-byte ,(1- sb-vm:n-word-bits)))
+                    (typep ,x '(integer ,(- 1 (expt 2 (- sb-vm:n-word-bits 1)))
+                                ,(1- (expt 2 (- sb-vm:n-word-bits 1))))))))
     (number-dispatch ((number real) (divisor real))
       ((fixnum fixnum) (truncate number divisor))
       (((foreach fixnum bignum) ratio)
@@ -330,16 +349,19 @@
                          (numerator divisor))
              (values quot (/ rem (denominator divisor))))))
       ((fixnum bignum)
-       (bignum-truncate (make-small-bignum number) divisor))
+       (if (single-digit-bignum-p divisor)
+           (bignum-truncate-single-digit (make-small-bignum number) divisor)
+           (bignum-truncate (make-small-bignum number) divisor)))
       ((ratio (or float rational))
        (let ((q (truncate (numerator number)
                           (* (denominator number) divisor))))
          (values q (- number (* q divisor)))))
       ((bignum fixnum)
-       (bignum-truncate number (make-small-bignum divisor)))
+       (bignum-truncate-single-digit number divisor))
       ((bignum bignum)
-       (bignum-truncate number divisor))
-
+       (if (single-digit-bignum-p divisor)
+           (bignum-truncate-single-digit number divisor)
+           (bignum-truncate number divisor)))
       (((foreach single-float double-float #+long-float long-float)
         (or rational single-float))
        (if (eql divisor 1)
