@@ -28,6 +28,24 @@
 (define-move-fun (store-double 2) (vop x y)
                  ((double-reg) (double-stack))
   (storew x (current-nfp-tn vop) (tn-offset y)))
+
+(define-move-fun (load-fp-immediate 1) (vop x y)
+                 ((single-immediate) (single-reg)
+                  (double-immediate) (double-reg))
+  (let ((x (tn-value x)))
+    (cond ((or (eql x $0f0)
+               (eql x $0d0))
+           (inst fmov y zr-tn))
+          ((encode-fp-immediate  x)
+           (inst fmov y x))
+          ((load-immediate-word tmp-tn (if (double-float-p x)
+                                           (double-float-bits x)
+                                           (single-float-bits x))
+                                t)
+           (inst fmov y tmp-tn))
+          (t
+           (load-inline-constant y x)))))
+
 
 ;;;; Move VOPs:
 
@@ -63,15 +81,13 @@
   (:args (x :scs (double-reg) :to :save))
   (:results (y))
   (:note "float to pointer coercion")
-  (:temporary (:sc non-descriptor-reg) flag)
-  (:temporary (:scs (interior-reg)) lip)
+  (:temporary (:scs (non-descriptor-reg) :offset lr-offset) lr)
   (:results (y :scs (descriptor-reg)))
   (:generator 13
-   (with-fixed-allocation (y flag
-                           double-float-widetag
-                           double-float-value-slot
-                           :lip lip)
-     (storew x y double-float-value-slot other-pointer-lowtag))))
+    (with-fixed-allocation (y lr
+                            double-float-widetag
+                            double-float-value-slot)
+      (storew x y double-float-value-slot other-pointer-lowtag))))
 
 (define-move-vop move-from-double :move (double-reg) (descriptor-reg))
 
@@ -144,7 +160,7 @@
     (:results (y :scs (complex-single-reg) :load-if (not (location= x y))))
     (:note "complex single float move")
     (:generator 0
-      (inst s-mov y x)))
+      (move-complex-double y x)))
 (define-move-vop complex-single-move :move
   (complex-single-reg) (complex-single-reg))
 
@@ -154,7 +170,7 @@
   (:results (y :scs (complex-double-reg) :load-if (not (location= x y))))
   (:note "complex double float move")
   (:generator 0
-    (inst s-mov y x)))
+    (move-complex-double y x)))
 (define-move-vop complex-double-move :move
   (complex-double-reg) (complex-double-reg))
 
@@ -166,34 +182,31 @@
 (define-vop (move-from-complex-single)
   (:args (x :scs (complex-single-reg) :to :save))
   (:results (y :scs (descriptor-reg)))
-  (:temporary (:sc non-descriptor-reg) pa-flag)
-  (:temporary (:scs (interior-reg)) lip)
+  (:temporary (:scs (non-descriptor-reg) :offset lr-offset) lr)
   (:note "complex single float to pointer coercion")
   (:generator 13
-     (with-fixed-allocation (y pa-flag complex-single-float-widetag
-                             complex-single-float-size :lip lip)
-       (inst str x (@ y (- (* complex-single-float-data-slot
-                              n-word-bytes)
-                           other-pointer-lowtag))))))
+    (with-fixed-allocation (y lr complex-single-float-widetag
+                            complex-single-float-size)
+      (inst str x (@ y (- (* complex-single-float-data-slot
+                             n-word-bytes)
+                          other-pointer-lowtag))))))
 (define-move-vop move-from-complex-single :move
   (complex-single-reg) (descriptor-reg))
 
 (define-vop (move-from-complex-double)
   (:args (x :scs (complex-double-reg) :to :save))
   (:results (y :scs (descriptor-reg)))
-  (:temporary (:sc non-descriptor-reg) pa-flag)
-  (:temporary (:scs (interior-reg)) lip)
+  (:temporary (:scs (non-descriptor-reg) :offset lr-offset) lr)
   (:note "complex double float to pointer coercion")
   (:generator 13
-     (with-fixed-allocation (y pa-flag complex-double-float-widetag
-                             complex-double-float-size :lip lip)
-       (inst str x (@ y (- (* complex-double-float-real-slot
-                              n-word-bytes)
-                           other-pointer-lowtag))))))
+    (with-fixed-allocation (y lr complex-double-float-widetag
+                            complex-double-float-size)
+      (inst str x (@ y (- (* complex-double-float-real-slot
+                             n-word-bytes)
+                          other-pointer-lowtag))))))
 (define-move-vop move-from-complex-double :move
   (complex-double-reg) (descriptor-reg))
 
-
 ;;;
 ;;; Move from a descriptor to a complex float register
 ;;;
@@ -243,7 +256,7 @@
   (:generator 2
     (sc-case y
       (complex-double-reg
-       (inst s-mov y x))
+       (move-complex-double y x))
       (complex-double-stack
        (storew x nfp (tn-offset y))))))
 (define-move-vop move-complex-double-float-arg :move-arg
@@ -582,7 +595,7 @@
       (double-stack
         (inst ldr bits
               (@ (current-nfp-tn vop)
-                 (load-store-offset (* (tn-offset float) n-word-bytes)))))
+                 (load-store-offset (tn-byte-offset float)))))
       (descriptor-reg
        (inst ldr bits
              (@ float (- (* double-float-value-slot n-word-bytes)
@@ -605,7 +618,7 @@
         (inst ldr (32-bit-reg hi-bits)
               (@ (current-nfp-tn vop)
                  (load-store-offset
-                  (+ (* (tn-offset float) n-word-bytes)
+                  (+ (tn-byte-offset float)
                      (if (eq *backend-byte-order* :big-endian)
                          0
                          4))))))
@@ -636,7 +649,7 @@
        (inst ldr (32-bit-reg lo-bits)
              (@ (current-nfp-tn vop)
                 (load-store-offset
-                 (+ (* (tn-offset float) n-word-bytes)
+                 (+ (tn-byte-offset float)
                     (if (eq *backend-byte-order* :big-endian)
                         4
                         0))))))
@@ -694,7 +707,7 @@
       (complex-single-reg
        (unless (eql (tn-offset r) (tn-offset real))
          (inst s-mov r real))
-       (inst s-ins r 1 imag 0 :s))
+       (inst ins r 1 imag 0 :s))
       (complex-single-stack
        (let ((nfp (current-nfp-tn vop))
              (offset (tn-offset r)))
@@ -725,7 +738,7 @@
       (complex-double-reg
        (unless (eql (tn-offset r) (tn-offset real))
          (inst s-mov r real))
-       (inst s-ins r 1 imag 0 :d))
+       (inst ins r 1 imag 0 :d))
       (complex-double-stack
        (let ((nfp (current-nfp-tn vop))
              (offset (tn-offset r)))
@@ -750,7 +763,7 @@
   (:generator 3
     (sc-case x
       (complex-single-reg
-       (inst s-ins r 0 x (ecase slot
+       (inst ins r 0 x (ecase slot
                            (:real 0)
                            (:imag 1))
              :s))
@@ -785,7 +798,7 @@
   (:generator 3
     (sc-case x
       (complex-double-reg
-       (inst s-ins r 0 x (ecase slot
+       (inst ins r 0 x (ecase slot
                            (:real 0)
                            (:imag 1))
              :d))
@@ -802,3 +815,33 @@
   (:translate imagpart)
   (:note "complex double float imagpart")
   (:variant :imag))
+
+(define-vop ()
+  (:translate round-double)
+  (:policy :fast-safe)
+  (:args (x :scs (double-reg) :target r))
+  (:arg-types double-float (:constant symbol))
+  (:info mode)
+  (:results (r :scs (double-reg)))
+  (:result-types double-float)
+  (:generator 2
+    (ecase mode
+      (:round (inst frintn r x))
+      (:floor (inst frintm r x))
+      (:ceiling (inst frintp r x))
+      (:truncate (inst frintz r x)))))
+
+(define-vop ()
+  (:translate round-single)
+  (:policy :fast-safe)
+  (:args (x :scs (single-reg) :target r))
+  (:arg-types single-float (:constant symbol))
+  (:info mode)
+  (:results (r :scs (single-reg)))
+  (:result-types single-float)
+  (:generator 2
+    (ecase mode
+      (:round (inst frintn r x))
+      (:floor (inst frintm r x))
+      (:ceiling (inst frintp r x))
+      (:truncate (inst frintz r x)))))
