@@ -18,20 +18,18 @@
 
 ;;;; register specs
 
-(eval-when (:compile-toplevel :load-toplevel :execute)
-  (defvar *register-names* (make-array 32 :initial-element nil)))
+(defvar *register-names* (make-array 32 :initial-element nil))
 
 (macrolet ((defreg (name offset)
              (let ((offset-sym (symbolicate name "-OFFSET")))
-               `(eval-when (:compile-toplevel :load-toplevel :execute)
+               `(progn
                   (defconstant ,offset-sym ,offset)
                   (setf (svref *register-names* ,offset-sym) ,(symbol-name name)))))
 
            (defregset (name &rest regs)
-             `(eval-when (:compile-toplevel :load-toplevel :execute)
-                (defparameter ,name
+             `(defglobal ,name
                   (list ,@(mapcar #'(lambda (name)
-                                      (symbolicate name "-OFFSET")) regs))))))
+                                      (symbolicate name "-OFFSET")) regs)))))
 
   (defreg nl0 0)
   (defreg nl1 1)
@@ -81,7 +79,7 @@
       r0 r1 r2 r3 r4 r5 r6 r7 r8 r9 #-darwin r10 #-sb-thread r11 lexenv)
 
   (defregset non-descriptor-regs
-      nl0 nl1 nl2 nl3 nl4 nl5 nl6 nl7 nl8 nl9 nargs nfp ocfp)
+      nl0 nl1 nl2 nl3 nl4 nl5 nl6 nl7 nl8 nl9 nargs nfp ocfp lr)
 
   (defregset boxed-regs
       r0 r1 r2 r3 r4 r5 r6
@@ -93,7 +91,10 @@
   (defconstant register-arg-count 4)
   ;; names and offsets for registers used to pass arguments
   (defregset *register-arg-offsets*  r0 r1 r2 r3)
-  (defparameter *register-arg-names* '(r0 r1 r2 r3)))
+  (defparameter *register-arg-names* '(r0 r1 r2 r3))
+  (defregset *descriptor-args* r0 r1 r2 r3 r4 r5 r6 r7 r8 r9)
+  (defregset *non-descriptor-args* nl0 nl1 nl2 nl3 nl4 nl5 nl6 nl7 nl8 nl9)
+  (defglobal *float-regs* (loop for i below 32 collect i)))
 
 
 ;;;; SB and SC definition:
@@ -114,7 +115,6 @@
   ;; Anything else that can be an immediate.
   (immediate immediate-constant)
 
-
   ;; **** The stacks.
 
   ;; The control stack.  (Scanned by GC)
@@ -186,23 +186,19 @@
   (non-descriptor-reg registers
                       :locations #.non-descriptor-regs)
 
-  ;; Pointers to the interior of objects.  Used only as a temporary.
-  (interior-reg registers
-                :locations (#.lr-offset))
-
   ;; **** Things that can go in the floating point registers.
 
-  ;; Non-Descriptor single-floats.
+  (single-immediate immediate-constant)
+  (double-immediate immediate-constant)
+
   (single-reg float-registers
               :locations #.(loop for i below 32 collect i)
-              :constant-scs ()
+              :constant-scs (single-immediate)
               :save-p t
               :alternate-scs (single-stack))
-
-  ;; Non-Descriptor double-floats.
   (double-reg float-registers
               :locations #.(loop for i below 32 collect i)
-              :constant-scs ()
+              :constant-scs (double-immediate)
               :save-p t
               :alternate-scs (double-stack))
 
@@ -219,7 +215,8 @@
                       :alternate-scs (complex-double-stack))
 
   (catch-block control-stack :element-size catch-block-size)
-  (unwind-block control-stack :element-size unwind-block-size))
+  (unwind-block control-stack :element-size unwind-block-size)
+  (zero immediate-constant))
 
 ;;;; Make some random tns for important registers.
 
@@ -241,9 +238,9 @@
   (defregtn zr any-reg)
   (defregtn cfp any-reg)
   (defregtn csp any-reg)
-  (defregtn lr interior-reg)
+  (defregtn lr any-reg)
   #+sb-thread
-  (defregtn thread interior-reg))
+  (defregtn thread any-reg))
 
 ;;; If VALUE can be represented as an immediate constant, then return the
 ;;; appropriate SC number, otherwise return NIL.
@@ -257,7 +254,11 @@
     (symbol
      (if (static-symbol-p value)
          immediate-sc-number
-         nil))))
+         nil))
+    (double-float
+     double-immediate-sc-number)
+    (single-float
+     single-immediate-sc-number)))
 
 (defun boxed-immediate-sc-p (sc)
   (eql sc immediate-sc-number))
@@ -336,10 +337,10 @@
                      (destructuring-bind (size posn integer)
                          (sb-c::basic-combination-args node)
                        (declare (ignore integer))
-                       (and (plusp (sb-c::lvar-value posn))
-                            (or (= (sb-c::lvar-value size) 1)
-                                (<= (+ (sb-c::lvar-value size)
-                                       (sb-c::lvar-value posn))
+                       (and (plusp (sb-c:lvar-value posn))
+                            (or (= (sb-c:lvar-value size) 1)
+                                (<= (+ (sb-c:lvar-value size)
+                                       (sb-c:lvar-value posn))
                                     n-word-bits)))))))
          (if (or (validp 'word)
                  (validp 'signed-word))
@@ -387,4 +388,6 @@
         sb-arm64-asm::encode-logical-immediate
         sb-arm64-asm::fixnum-encode-logical-immediate
         bic-encode-immediate
-        bic-fixnum-encode-immediate))
+        bic-fixnum-encode-immediate
+        logical-immediate-or-word-mask
+        sb-arm64-asm::ldr-str-offset-encodable))
