@@ -19,7 +19,7 @@
     ;; Note:
     ;; This is not FLUSHABLE because it's defined to signal errors.
     (movable)
-  ;; :DERIVE-TYPE RESULT-TYPE-SPEC-NTH-ARG 2 ? Nope... (COERCE 1 'COMPLEX)
+  ;; :DERIVE-TYPE RESULT-TYPE-SPEC-NTH-ARG 1 ? Nope... (COERCE 1 'COMPLEX)
   ;; returns REAL/INTEGER, not COMPLEX.
   )
 ;; These each check their input sequence for type-correctness,
@@ -103,6 +103,7 @@
 #+64-bit (defknown layout-depthoid (sb-vm:layout) layout-depthoid (flushable always-translatable))
 #+(or x86 x86-64) (defknown (layout-depthoid-ge) (sb-vm:layout integer) boolean (flushable))
 (defknown %structure-is-a (instance t) boolean (foldable flushable))
+(defknown structure-typep (t t) boolean (foldable flushable))
 (defknown copy-structure (structure-object) structure-object
   (flushable)
   :derive-type #'result-type-first-arg)
@@ -344,6 +345,30 @@
   (real &optional real) (values integer real)
   (movable foldable flushable recursive))
 
+(defknown unary-truncate (real) (values integer real)
+  (movable foldable flushable no-verify-arg-count))
+
+(defknown unary-truncate-single-float-to-bignum (single-float) (values bignum (eql $0f0))
+   (foldable movable flushable fixed-args))
+(defknown unary-truncate-double-float-to-bignum (double-float)
+    (values #+64-bit bignum #-64-bit integer
+            (and
+             #+(and 64-bit
+                    (not (or riscv ppc64))) ;; they can't survive cold-init
+             (eql $0d0)
+             double-float))
+   (foldable movable flushable fixed-args))
+
+(defknown %unary-truncate-single-float-to-bignum (single-float) bignum
+   (foldable movable flushable fixed-args))
+(defknown %unary-truncate-double-float-to-bignum (double-float) bignum
+   (foldable movable flushable fixed-args))
+
+(defknown sxhash-bignum-double-float (double-float) hash-code
+  (foldable movable flushable fixed-args))
+(defknown sxhash-bignum-single-float (single-float) hash-code
+  (foldable movable flushable fixed-args))
+
 (defknown %multiply-high (word word) word
     (movable foldable flushable))
 
@@ -517,15 +542,15 @@
   (flushable))
 
 (defknown copy-seq (proper-sequence) consed-sequence (flushable)
-  :derive-type (sequence-result-nth-arg 1 :preserve-dimensions t))
+  :derive-type (sequence-result-nth-arg 0 :preserve-dimensions t))
 
 (defknown length (proper-sequence) index (foldable flushable dx-safe))
 
 (defknown reverse (proper-sequence) consed-sequence (flushable)
-  :derive-type (sequence-result-nth-arg 1 :preserve-dimensions t))
+  :derive-type (sequence-result-nth-arg 0 :preserve-dimensions t))
 
 (defknown nreverse ((modifying sequence)) sequence (important-result)
-  :derive-type (sequence-result-nth-arg 1 :preserve-dimensions t
+  :derive-type (sequence-result-nth-arg 0 :preserve-dimensions t
                                           :preserve-vector-type t))
 
 (defknown make-sequence (type-specifier index
@@ -533,10 +558,10 @@
                                         (:initial-element t))
   consed-sequence
   (movable)
-  :derive-type (creation-result-type-specifier-nth-arg 1))
+  :derive-type (creation-result-type-specifier-nth-arg 0))
 
 (defknown concatenate (type-specifier &rest proper-sequence) consed-sequence ()
-  :derive-type (creation-result-type-specifier-nth-arg 1))
+  :derive-type (creation-result-type-specifier-nth-arg 0))
 
 (defknown %concatenate-to-string (&rest sequence) simple-string
   (flushable no-verify-arg-count))
@@ -570,7 +595,8 @@
                     &rest proper-sequence)
   sequence
   (call)
-  :derive-type #'result-type-first-arg)
+  :derive-type (sequence-result-nth-arg 0 :preserve-dimensions t
+                                          :preserve-vector-type t))
 
 (defknown #.(loop for info across sb-vm:*specialized-array-element-type-properties*
                   collect
@@ -601,7 +627,8 @@
 (defknown fill ((modifying sequence) t &rest t &key
                 (:start index) (:end sequence-end)) sequence
     ()
-  :derive-type #'result-type-first-arg
+  :derive-type (sequence-result-nth-arg 0 :preserve-dimensions t
+                                          :preserve-vector-type t)
   :result-arg 0)
 ;;; Like FILL but with no keyword argument parsing
 (defknown quickfill ((modifying (simple-array * 1)) t) (simple-array * 1) ()
@@ -620,7 +647,8 @@
 (defknown replace ((modifying sequence) proper-sequence &rest t &key (:start1 index)
                    (:end1 sequence-end) (:start2 index) (:end2 sequence-end))
   sequence ()
-  :derive-type #'result-type-first-arg
+  :derive-type (sequence-result-nth-arg 0 :preserve-dimensions t
+                                          :preserve-vector-type t)
   :result-arg 0)
 
 (defknown remove
@@ -633,7 +661,7 @@
      (:key (function-designator ((nth-arg 1 :sequence t)))))
   consed-sequence
   (flushable call)
-  :derive-type (sequence-result-nth-arg 2))
+  :derive-type (sequence-result-nth-arg 1))
 
 (defknown substitute
   (t t proper-sequence &rest t &key (:from-end t)
@@ -645,7 +673,7 @@
      (:key (function-designator ((nth-arg 2 :sequence t)))))
   consed-sequence
   (flushable call)
-  :derive-type (sequence-result-nth-arg 3))
+  :derive-type (sequence-result-nth-arg 2))
 
 (defknown (remove-if remove-if-not)
   ((function-designator ((nth-arg 1 :sequence t :key :key))) proper-sequence
@@ -656,7 +684,7 @@
    (:key (function-designator ((nth-arg 1 :sequence t)))))
   consed-sequence
   (flushable call)
-  :derive-type (sequence-result-nth-arg 2))
+  :derive-type (sequence-result-nth-arg 1))
 
 (defknown (substitute-if substitute-if-not)
   (t (function-designator ((nth-arg 2 :sequence t :key :key))) proper-sequence
@@ -667,7 +695,7 @@
      (:key (function-designator ((nth-arg 2 :sequence t)))))
   consed-sequence
   (flushable call)
-  :derive-type (sequence-result-nth-arg 3))
+  :derive-type (sequence-result-nth-arg 2))
 
 (defknown delete
   (t (modifying sequence) &rest t &key (:from-end t)
@@ -678,7 +706,7 @@
      (:key (function-designator ((nth-arg 1 :sequence t)))))
   sequence
   (call important-result)
-  :derive-type (sequence-result-nth-arg 2))
+  :derive-type (sequence-result-nth-arg 1))
 
 (defknown nsubstitute
   (t t (modifying sequence) &rest t &key (:from-end t)
@@ -689,7 +717,7 @@
      (:key (function-designator ((nth-arg 2 :sequence t)))))
   sequence
   (call)
-  :derive-type (sequence-result-nth-arg 3))
+  :derive-type (sequence-result-nth-arg 2))
 
 (defknown (delete-if delete-if-not)
   ((function-designator ((nth-arg 1 :sequence t :key :key))) (modifying sequence)
@@ -698,7 +726,7 @@
    (:key (function-designator ((nth-arg 1 :sequence t)))))
   sequence
   (call important-result)
-  :derive-type (sequence-result-nth-arg 2))
+  :derive-type (sequence-result-nth-arg 1))
 
 (defknown (nsubstitute-if nsubstitute-if-not)
   (t (function-designator ((nth-arg 2 :sequence t :key :key))) (modifying sequence)
@@ -707,7 +735,7 @@
      (:key (function-designator ((nth-arg 2 :sequence t)))))
   sequence
   (call)
-  :derive-type (sequence-result-nth-arg 3))
+  :derive-type (sequence-result-nth-arg 2))
 
 (defknown remove-duplicates
   (proper-sequence &rest t &key
@@ -721,7 +749,7 @@
             (:key (function-designator ((nth-arg 0 :sequence t)))))
   consed-sequence
   (flushable call)
-  :derive-type (sequence-result-nth-arg 1))
+  :derive-type (sequence-result-nth-arg 0))
 
 (defknown delete-duplicates
   ((modifying sequence)
@@ -735,7 +763,7 @@
    (:key (function-designator ((nth-arg 0 :sequence t)))))
   sequence
   (call important-result)
-  :derive-type (sequence-result-nth-arg 1))
+  :derive-type (sequence-result-nth-arg 0))
 
 (defknown find
   (t proper-sequence &rest t &key
@@ -833,7 +861,8 @@
    &key (:key (function-designator ((nth-arg 0 :sequence t)))))
   sequence
   (call)
-  :derive-type #'result-type-first-arg)
+  :derive-type (sequence-result-nth-arg 0 :preserve-dimensions t
+                                          :preserve-vector-type t))
 (defknown sb-impl::stable-sort-list (list function function) list
   (call important-result no-verify-arg-count))
 (defknown sb-impl::sort-vector (vector index index function (or function null))
@@ -857,7 +886,7 @@
                                                      (nth-arg 2 :sequence t))))))
   sequence
   (call important-result)
-  :derive-type (creation-result-type-specifier-nth-arg 1))
+  :derive-type (creation-result-type-specifier-nth-arg 0))
 
 (defknown read-sequence ((modifying sequence) stream
                          &key
@@ -1260,7 +1289,8 @@
   (string-designator &key
    (:start (inhibit-flushing index 0))
    (:end (inhibit-flushing sequence-end nil)))
-  simple-string (flushable))
+  simple-string (flushable)
+  :derive-type (sequence-result-nth-arg 0 :preserve-dimensions t))
 
 (defknown (nstring-upcase nstring-downcase nstring-capitalize)
   ((modifying string) &key (:start index) (:end sequence-end))
@@ -1891,10 +1921,10 @@
 (defknown data-vector-set (array index t) (values) (dx-safe always-translatable))
 (defknown data-vector-set-with-offset (array fixnum fixnum t) (values)
   (dx-safe always-translatable))
-(defknown hairy-data-vector-ref (array index) t (foldable))
-(defknown hairy-data-vector-set (array index t) t ())
-(defknown hairy-data-vector-ref/check-bounds (array index) t (foldable))
-(defknown hairy-data-vector-set/check-bounds (array index t) t ())
+(defknown hairy-data-vector-ref (array index) t (foldable no-verify-arg-count))
+(defknown hairy-data-vector-set (array index t) t (no-verify-arg-count))
+(defknown hairy-data-vector-ref/check-bounds (array index) t (foldable no-verify-arg-count))
+(defknown hairy-data-vector-set/check-bounds (array index t) t (no-verify-arg-count))
 (defknown %caller-frame () t (flushable))
 (defknown %caller-pc () system-area-pointer (flushable))
 (defknown %with-array-data (array index (or index null))
@@ -2022,7 +2052,6 @@
 (defknown %alien-funcall ((or string system-area-pointer) alien-type &rest t) *)
 
 ;; Used by WITH-PINNED-OBJECTS
-#+(or x86 x86-64)
 (defknown sb-vm::touch-object (t) (values)
   (always-translatable))
 
