@@ -78,11 +78,9 @@
   (:info name offset lowtag)
   (:temporary (:scs (non-descriptor-reg)) t1)
   (:vop-var vop)
+  (:ignore name)
   (:generator 1
-    ;; gencgc does not need to emit the barrier for constructors
-    (unless (member name '(%make-structure-instance make-weak-pointer
-                           %make-ratio %make-complex))
-      (emit-gc-store-barrier object nil (list t1) (vop-nth-arg 1 vop) value))
+    (emit-gc-store-barrier object nil (list t1) (vop-nth-arg 1 vop) value)
     (storew value object offset lowtag)))
 
 (define-vop (compare-and-swap-slot)
@@ -129,7 +127,7 @@
     (inst bne DONT-STORE-TLS)
     (inst stdx new thread-base-tn temp)
     DONT-STORE-TLS
-    (inst cmpdi result no-tls-value-marker-widetag)
+    (inst cmpdi result no-tls-value-marker)
     (inst bne CHECK-UNBOUND)
 
     (inst li temp (- (* symbol-value-slot n-word-bytes)
@@ -199,7 +197,7 @@
   (:generator 4
     (load-tls-index tls-slot symbol)
     (inst ldx temp thread-base-tn tls-slot)
-    (inst cmpdi temp no-tls-value-marker-widetag)
+    (inst cmpdi temp no-tls-value-marker)
     (inst beq GLOBAL-VALUE)
     (inst stdx value thread-base-tn tls-slot)
     (inst b DONE)
@@ -221,7 +219,7 @@
     (inst beq NULL)
     (load-tls-index value object)
     (inst ldx value thread-base-tn value)
-    (inst cmpdi value no-tls-value-marker-widetag)
+    (inst cmpdi value no-tls-value-marker)
     (inst bne CHECK-UNBOUND)
     (loadw value object symbol-value-slot other-pointer-lowtag)
     CHECK-UNBOUND
@@ -245,7 +243,7 @@
     (inst beq NULL)
     (load-tls-index value object)
     (inst ldx value thread-base-tn value)
-    (inst cmpdi value no-tls-value-marker-widetag)
+    (inst cmpdi value no-tls-value-marker)
     (inst bne DONE)
     (loadw value object symbol-value-slot other-pointer-lowtag)
     (inst b DONE)
@@ -267,7 +265,7 @@
     (inst beq (if not-p out target))
     (load-tls-index value object)
     (inst ldx value thread-base-tn value)
-    (inst cmpdi value no-tls-value-marker-widetag)
+    (inst cmpdi value no-tls-value-marker)
     (inst bne CHECK-UNBOUND)
     (loadw value object symbol-value-slot other-pointer-lowtag)
     CHECK-UNBOUND
@@ -459,7 +457,9 @@
   (:args (object :scs (descriptor-reg))
          (value :scs (descriptor-reg any-reg)))
   (:info offset)
+  (:temporary (:scs (non-descriptor-reg)) temp)
   (:generator 4
+    (emit-gc-store-barrier object nil (list temp))
     (storew value object (+ closure-info-offset offset) fun-pointer-lowtag)))
 
 (define-vop (closure-init-from-fp)
