@@ -133,8 +133,8 @@
 
 
 (deftransform %alien-funcall ((function type &rest args) * * :node node)
-  (aver (sb-c::constant-lvar-p type))
-  (let* ((type (sb-c::lvar-value type))
+  (aver (sb-c:constant-lvar-p type))
+  (let* ((type (sb-c:lvar-value type))
          (env (sb-c::node-lexenv node))
          (arg-types (alien-fun-type-arg-types type))
          (result-type (alien-fun-type-result-type type)))
@@ -201,9 +201,8 @@
     (foldable flushable movable))
 
 (defoptimizer (sign-extend derive-type) ((x size))
-  (declare (ignore x))
-  (when (sb-c::constant-lvar-p size)
-    (specifier-type `(signed-byte ,(sb-c::lvar-value size)))))
+  (when (sb-c:constant-lvar-p size)
+    (specifier-type `(signed-byte ,(sb-c:lvar-value size)))))
 
 (define-vop (sign-extend)
   (:translate sign-extend)
@@ -264,10 +263,10 @@
       (append
        (loop for gpr in gprs
              collect `(:temporary (:sc any-reg :offset ,gpr :from :eval :to :result)
-                                  ,(car (push (sb-xc:gensym) vars))))
+                                  ,(car (push (gensym) vars))))
        (loop for float to 15
              collect `(:temporary (:sc single-reg :offset ,float :from :eval :to :result)
-                                  ,(car (push (sb-xc:gensym) vars))))
+                                  ,(car (push (gensym) vars))))
        `((:ignore ,@vars))))))
 
 (define-vop (call-out)
@@ -357,7 +356,7 @@
   ;; Store SP in thread struct, unless the enclosing block says not to
   #+sb-safepoint
   (when (policy (sb-c::vop-node vop) (/= sb-c:insert-safepoints 0))
-    (storew rsp-tn thread-base-tn thread-saved-csp-offset))
+    (inst mov (thread-slot-ea thread-saved-csp-offset) rsp-tn))
 
   #+win32 (inst sub rsp-tn #x20)       ;MS_ABI: shadow zone
 
@@ -391,8 +390,7 @@
   ;; Zero the saved CSP, unless this code shouldn't ever stop for GC
   #+sb-safepoint
   (when (policy (sb-c::vop-node vop) (/= sb-c:insert-safepoints 0))
-    (inst xor (object-slot-ea thread-base-tn thread-saved-csp-offset 0)
-          rsp-tn)))
+    (inst xor (thread-slot-ea thread-saved-csp-offset) rsp-tn)))
 
 (define-vop (alloc-number-stack-space)
   (:info amount)
@@ -421,17 +419,6 @@
           (inst sub :qword (alien-stack-ptr) delta)))
       (inst mov result (alien-stack-ptr)))))
 
-;;; not strictly part of the c-call convention, but needed for the
-;;; WITH-PINNED-OBJECTS macro used for "locking down" lisp objects so
-;;; that GC won't move them while foreign functions go to work.
-(define-vop (touch-object)
-  (:translate touch-object)
-  (:args (object))
-  (:ignore object)
-  (:policy :fast-safe)
-  (:arg-types t)
-  (:generator 0))
-
 ;;; Callbacks
 
 #-sb-xc-host
@@ -488,7 +475,7 @@
                      ;; stack location to a temporary register.
                      (unless gpr
                        (incf stack-argument-count)
-                       (setf gpr temp-reg-tn)
+                       (setf gpr rax)
                        (inst mov gpr stack-arg-tn))
                      ;; Copy from either argument register or temporary
                      ;; register to target.
@@ -505,8 +492,8 @@
                             ;; temporary (general purpose) register, and
                             ;; from there to the target location.
                             (incf stack-argument-count)
-                            (inst mov temp-reg-tn stack-arg-tn)
-                            (inst mov target-tn temp-reg-tn)))))
+                            (inst mov rax stack-arg-tn)
+                            (inst mov target-tn rax)))))
                   (t
                    (bug "Unknown alien floating point type: ~S" type)))))
 
