@@ -117,11 +117,19 @@ os_context_sigmask_addr(os_context_t *context)
 }
 
 os_vm_address_t
-os_validate(int attributes, os_vm_address_t addr, os_vm_size_t len, int executable, int jit)
+os_validate(int attributes, os_vm_address_t addr, os_vm_size_t len, int space_id)
 {
+    int __attribute((unused))
+      executable = (space_id == READ_ONLY_CORE_SPACE_ID) ||
+                   (space_id == LINKAGE_TABLE_CORE_SPACE_ID) ||
+                   (space_id == STATIC_CODE_CORE_SPACE_ID),
+      jit = (space_id == STATIC_CODE_CORE_SPACE_ID) || (space_id == DYNAMIC_CORE_SPACE_ID)
+            ? 1 : (space_id == LINKAGE_TABLE_CORE_SPACE_ID) ? 2 : 0;
+
     int protection;
     int flags = 0;
 
+    // FIXME: This probaby needs to use MAP_TRYFIXED
     if (attributes & IS_GUARD_PAGE)
         protection = OS_VM_PROT_NONE;
     else
@@ -273,7 +281,7 @@ memory_fault_handler(int signal, siginfo_t *siginfo, os_context_t *context)
     if (handle_safepoint_violation(context, fault_addr)) return;
 #endif
 
-    if (gencgc_handle_wp_violation(fault_addr)) return;
+    if (gencgc_handle_wp_violation(context, fault_addr)) return;
 
     if (!handle_guard_page_triggered(context,fault_addr))
             lisp_memory_fault_error(context, fault_addr);
@@ -290,7 +298,6 @@ mach_error_memory_fault_handler(int signal, siginfo_t *siginfo,
 void
 os_install_interrupt_handlers(void)
 {
-    SHOW("os_install_interrupt_handlers()/bsd-os/defined(GENCGC)");
     if (INSTALL_SIG_MEMORY_FAULT_HANDLER) {
 #if defined(LISP_FEATURE_MACH_EXCEPTION_HANDLER)
     ll_install_handler(SIG_MEMORY_FAULT, mach_error_memory_fault_handler);
